/**
 * @file staffldr.cpp
 * @brief Contans the load functions for the staff's built-in test scenes.
 *
 * @author Eric Butler (edbutler)
 * @author Zeyang Li (zeyangl)
 */

/*
    DO NOT MODIFY THIS FILE. It contains the staff test scnes and will be
    updated with each project handout. If you want to add your own scenes,
    do so elsewhere. The suggested file for that is ldr.cpp.
 */

#include "project.h"
#include "scene.h"
#include "geom/sphere.h"
#include "geom/triangle.h"
#include "geom/watersurface.h"
#include <iostream>

static void create_square(Scene* scene, const Vec3& xaxis, const Vec3& yaxis,
                          const Vec3& corner, const Vec3& normal, Material* mat,
                          const Vec2& tcoord_min, const Vec2& tcoord_unit, Effect* effect)
{
    Vec3 maxi = corner + xaxis + yaxis;

    Vec3 vertices[3];
    Vec3 normals[] = {normal, normal, normal};
    Material* materials[] = { mat, mat, mat };
    Vec2 tcoords[3];

#ifdef UV_CLOCKWISE
    Vec2 dist(xaxis.magnitude(), yaxis.magnitude());	
#else
	// computing uvs in counterclockwise order
	Vec2 dist(yaxis.magnitude(), xaxis.magnitude());
#endif

    vertices[0] = maxi;
    vertices[1] = corner;
    vertices[2] = corner + yaxis;
	
    tcoords[0] = tcoord_min + dist * tcoord_unit;
    tcoords[1] = tcoord_min;
#ifdef UV_CLOCKWISE
    tcoords[2] = tcoord_min + Vec2(0, dist.y) * tcoord_unit;
#else
	tcoords[2] = tcoord_min + Vec2(dist.x, 0) * tcoord_unit;
#endif

    scene->objects.push_back(
        new Triangle(Vec3::Zero, Quat::Identity, Vec3::Ones,
                     vertices, tcoords, normals, materials, effect));

    vertices[0] = corner;
    vertices[1] = maxi;
    vertices[2] = corner + xaxis;
	
    tcoords[0] = tcoord_min;
    tcoords[1] = tcoord_min + dist * tcoord_unit;
#ifdef UV_CLOCKWISE
    tcoords[2] = tcoord_min + Vec2(dist.x, 0) * tcoord_unit;
#else
	tcoords[2] = tcoord_min + Vec2(0, dist.y) * tcoord_unit;
#endif

    scene->objects.push_back(
        new Triangle(Vec3::Zero, Quat::Identity, Vec3::Ones,
                     vertices, tcoords, normals, materials, effect));

}

#define SIZE 4

static void ldr_load_scene00(Scene* scene)
{
    //"Cornell Box" Scene
    Camera& cam = scene->camera;
    cam.orientation = Quat::Identity;
    cam.position = Vec3(0,2,9.2);
    cam.focus_dist = sqrt(4+9.2*9.2);
    cam.fov = PI / 3.0;
    cam.near_clip = .1;
    cam.far_clip = 100.0;

    scene->ambient_light = Vec3(.1,.1,.1);
    scene->refraction_index = 1;

    Vec3 floor_corner = Vec3(-SIZE, -SIZE/2, -SIZE);
    Vec3 ceiling_corner = Vec3(SIZE, SIZE + SIZE/2, -SIZE);

    //Green
    Material* g_mat = new Material();
    g_mat->ambient = Vec3(0.08, 0.57, 0.09);
    g_mat->diffuse = Vec3(0.08, 0.57, 0.09); 
    g_mat->phong = Vec3::Zero;
    g_mat->shininess = 40;
    g_mat->specular = Vec3::Zero;
    g_mat->refraction_index = 0;
    scene->materials.push_back(g_mat);

    create_square(scene,
        Vec3(0,0,SIZE-ceiling_corner.z), Vec3(0, floor_corner.y - ceiling_corner.y,0), 
        ceiling_corner, Vec3::UnitX * -1, g_mat, Vec2::Ones, Vec2::Zero, 0);

    //Red
    Material* r_mat = new Material();
    r_mat->ambient = Vec3(0.78, 0.04, 0.04);
    r_mat->diffuse = Vec3(0.78, 0.04, 0.04); 
    r_mat->specular = Vec3::Zero;
    r_mat->refraction_index = 0;
    scene->materials.push_back(r_mat);

    create_square(scene,
        Vec3(0,0,SIZE-ceiling_corner.z), Vec3(0, ceiling_corner.y - floor_corner.y,0), 
        floor_corner, Vec3::UnitX, r_mat, Vec2::Ones, Vec2::Zero, 0);

    //White
    Material* w_mat = new Material();
    w_mat->ambient = Vec3::Ones;
    w_mat->diffuse = Vec3::Ones; 
    w_mat->specular = Vec3::Zero;
    w_mat->refraction_index = 0;
    scene->materials.push_back(w_mat);

    create_square(scene,
        Vec3(0,0,SIZE-ceiling_corner.z), Vec3(ceiling_corner.x - floor_corner.x,0,0), 
        floor_corner, Vec3::UnitY, w_mat, Vec2::Ones, Vec2::Zero, 0);
    create_square(scene,
        Vec3(ceiling_corner.x - floor_corner.x,0,0), Vec3(0, ceiling_corner.y - floor_corner.y,0), 
        floor_corner, Vec3::UnitZ, w_mat, Vec2::Ones, Vec2::Zero, 0);
    create_square(scene,
        Vec3(0,0,SIZE-ceiling_corner.z), Vec3(floor_corner.x - ceiling_corner.x,0,0), 
        ceiling_corner, Vec3::UnitY * -1, w_mat, Vec2::Ones, Vec2::Zero, 0);

    //Crystal
    //Meant to have the properties of crystal
    Material* c_mat = new Material();
    c_mat->ambient = Vec3::Ones;
    c_mat->diffuse = Vec3::Ones;
    c_mat->phong = Vec3::Ones;
    c_mat->shininess = 10;
    c_mat->specular = Vec3::Ones;
    c_mat->refraction_index = 1.6;
    scene->materials.push_back(c_mat);

    scene->objects.push_back(
        new Sphere(Vec3(1, -1, 1), Quat::Identity, Vec3(1,1,1), 1, c_mat));
    scene->caustic_generator = scene->objects.back();

    //Mirror
    Material* m_mat = new Material();
    m_mat->ambient = Vec3::Ones;
    m_mat->diffuse = Vec3::Zero;
    m_mat->specular = Vec3::Ones;
    m_mat->refraction_index = 0;
    scene->materials.push_back(m_mat);

    scene->objects.push_back(
        new Sphere(Vec3(-1, 0, -1.75), Quat::Identity, Vec3(1,1,1), 2, m_mat));

    Light light;
    light.position = Vec3(0, 4, 0);
    light.color = Vec3(.7, .7, .7);
    light.intensity = 50;
    scene->lights.push_back(light);
}

static void ldr_load_scene01(Scene* scene)
{
    // ball scene
    Camera& cam = scene->camera;
    cam.orientation = Quat(0.938069, -0.271216, -0.207945, -0.0568061);
    cam.position = Vec3(-0.720501,0.0676861,1.41367);
    cam.focus_dist = (cam.position-Vec3::UnitY).magnitude();
    cam.fov = PI / 3.0;
    cam.near_clip = .1;
    cam.far_clip = 100.0;

    scene->ambient_light = Vec3(.1,.1,.1);
    scene->refraction_index = 1;

    Material* mat;

    //Green
    mat = new Material();
    mat->ambient = Vec3::Ones;
    mat->diffuse = Vec3::Ones;
    mat->phong = Vec3::Zero;
    mat->shininess = 40;
    mat->specular = Vec3::Zero;
    mat->refraction_index = 0;
    mat->texture_name = "images/bricks_diffuse.png";
    scene->materials.push_back(mat);

    create_square(scene,
        Vec3(2,0,0), Vec3(0,0,2), 
        Vec3(-1,-1,-1), Vec3::UnitY, mat, Vec2::Zero, Vec2(.25,.25), 0);

    //Crystal
    //Meant to have the properties of crystal
    mat = new Material();
    mat->ambient = Vec3::Ones;
    mat->diffuse = Vec3::Ones;
    mat->phong = Vec3::Ones;
    mat->shininess = 10;
    mat->specular = Vec3::Ones;
    mat->refraction_index = 2.0;
    scene->materials.push_back(mat);

    scene->objects.push_back(
        new Sphere(Vec3(0, -.75, 0), Quat::Identity, Vec3::Ones, .25, mat));
    scene->caustic_generator = scene->objects.back();

    Light light;
    light.position = Vec3(4, 4, 1) / 2;
    light.color = Vec3(.7, .7, .7);
    light.intensity = 50;
    scene->lights.push_back(light);
}


// pool boundaries
#define PIX 5
#define POX 9
#define PIZ 5
#define POZ 9
#define POY -.5
#define PIY -4
#define PBY -5

static void create_pool(Scene* scene)
{
    Material* mat = new Material();
    mat->ambient = Vec3(0.7, 0.7, 0.7);
    mat->diffuse = Vec3::Ones;
    mat->phong = Vec3::Ones;
    mat->shininess = 40;
    mat->specular = Vec3(.1,.1,.1);
    mat->refraction_index = 0;
	mat->texture_name = "images/marble.png";
    scene->materials.push_back(mat);

    Effect* efc = 0;

    Vec2 tcmin(0,0);
    Vec2 tcunit(.25,.25);

    // so very, very disgusting. please don't look, your eyes may melt

    // upper corners
    create_square(scene,
                  Vec3(POX-PIX,0,0), Vec3(0,0,2*POZ), Vec3(PIX,POY,-POZ),
                  Vec3::UnitY, mat, tcmin, tcunit, efc);
    create_square(scene,
                  Vec3(POX-PIX,0,0), Vec3(0,0,2*POZ), Vec3(-POX,POY,-POZ),
                  Vec3::UnitY, mat, tcmin, tcunit, efc);

    create_square(scene,
                  Vec3(2*PIX,0,0), Vec3(0,0,POZ-PIZ), Vec3(-PIX,POY,-POZ),
                  Vec3::UnitY, mat, tcmin, tcunit, efc);
    create_square(scene,
                  Vec3(2*PIX,0,0), Vec3(0,0,POZ-PIZ), Vec3(-PIX,POY,PIZ),
                  Vec3::UnitY, mat, tcmin, tcunit, efc);

    // inner sides

    create_square(scene,
                  Vec3(0,0,2*PIZ), Vec3(0,POY-PIY,0), Vec3(-PIX,PIY,-PIZ),
                  Vec3::UnitX, mat, tcmin, tcunit, efc);

    create_square(scene,
                  -Vec3(2*PIX,0,0), Vec3(0,POY-PIY,0), Vec3(PIX,PIY,-PIZ),
                  Vec3::UnitZ, mat, tcmin, tcunit, efc);

    create_square(scene,
                  -Vec3(0,POY-PIY,0), -Vec3(0,0,2*PIZ), Vec3(PIX,POY,PIZ),
                  -Vec3::UnitX, mat, tcmin, tcunit, efc);

    create_square(scene,
                  Vec3(2*PIX,0,0), Vec3(0,POY-PIY,0), Vec3(-PIX,PIY,PIZ),
                  -Vec3::UnitZ, mat, tcmin, tcunit, efc);

    // outer sides
    create_square(scene,
                  Vec3(0,POY-PBY,0), Vec3(0,0,2*POZ), Vec3(-POX,PBY,-POZ),
                  -Vec3::UnitX, mat, tcmin, tcunit, efc);

    create_square(scene,
                  Vec3(0,0,2*POZ), Vec3(0,POY-PBY,0), Vec3(POX,PBY,-POZ),
                  Vec3::UnitX, mat, tcmin, tcunit, efc);

    create_square(scene,
                  Vec3(2*POX,0,0), Vec3(0,POY-PBY,0), Vec3(-POX,PBY,-POZ),
                  -Vec3::UnitZ, mat, tcmin, tcunit, efc);

    create_square(scene,
                  Vec3(0,POY-PBY,0), Vec3(2*POX,0,0), Vec3(-POX,PBY,POZ),
                  Vec3::UnitZ, mat, tcmin, tcunit, efc);

    // bottom
    create_square(scene,
                  Vec3(0,0,2*PIZ), Vec3(2*PIX,0,0), Vec3(-PIX,PIY,-PIZ),
                  Vec3::UnitY, mat, tcmin, tcunit, efc);
    create_square(scene,
                  Vec3(0,0,2*POZ), Vec3(2*POX,0,0), Vec3(-POX,PBY,-POZ),
                  -Vec3::UnitY, mat, tcmin, tcunit, efc);

}

static void ldr_load_scene02(Scene* scene)
{
    // "Pool" Scene

    Camera& cam = scene->camera;
    cam.orientation = Quat(-0.0946664, -0.00690199, 0.970616, 0.22112);
    cam.position = Vec3(-2.62381,6.01017,-12.4194);
    cam.focus_dist = 14.0444;
    cam.fov = PI / 3.0;
    cam.near_clip = 1;
    cam.far_clip = 1000.0;

	// create sphere map
	SphereMap* spheremap = new SphereMap();
	scene->background = spheremap;
	spheremap->texture_name = "images/spheremap_stpeters.png";

    scene->ambient_light = Vec3(.2,.2,.2);
    scene->refraction_index = 1;

    create_pool(scene);

    Material* mat;

    mat = new Material();
    mat->ambient = Vec3(0.0, 0.2, 0.3);
    mat->diffuse = Vec3(0.0, 0.2, 0.3); // blue water
    mat->phong = Vec3::Ones;
    mat->shininess = 20;
    mat->specular = Vec3::Ones;
    mat->refraction_index = 1.33;
    scene->materials.push_back(mat);

    WaterSurface::WavePointList wave_points;
    WaterSurface::WavePoint* p;

    wave_points.push_back(WaterSurface::WavePoint());
    p = &wave_points.back();
    p->position = Vec2(.42,.56);
    p->falloff = 2;
    p->coefficient = .3;
    p->timerate = -6*PI;
    p->period = 16*PI;

    wave_points.push_back(WaterSurface::WavePoint());
    p = &wave_points.back();
    p->position = Vec2(-.58,-.30);
    p->falloff = 2;
    p->coefficient = .3;
    p->timerate = -8*PI;
    p->period = 20*PI;

    WaterSurface* water_surface = new WaterSurface(Vec3(0, POY - 1, 0),
                                     Quat::Identity,
                                     Vec3(PIX, 0.4, PIZ),
                                     wave_points,
                                     256, 256, mat);
    scene->objects.push_back(water_surface);
    scene->updatable_objects.push_back(water_surface);
    scene->caustic_generator = water_surface;

	Effect* fresnel = new FresnelEffect("shaders/fresnel_vert.glsl",
										"shaders/fresnel_frag.glsl",
										spheremap, mat);
	water_surface->effect = fresnel;
	scene->effects.push_back(fresnel);

    mat = new Material();
    mat->ambient = Vec3::Ones;
    mat->diffuse = Vec3::Ones;
    mat->phong = Vec3(1,.5,1);
    mat->shininess = 20;
    mat->specular = Vec3(.6,.6,.6);
    mat->refraction_index = 0;
    mat->texture_name = "images/swirly.png";
    scene->materials.push_back(mat);

    real_t rad = 2;
    scene->objects.push_back(
        new Sphere(Vec3((POX+PIX)/2, POY+rad, (POZ+PIZ)/2),
                   Quat::Identity, Vec3::Ones, rad, mat));
    scene->objects.push_back(
        new Sphere(Vec3(-(POX+PIX)/2, POY+rad, -(POZ+PIZ)/2),
                   Quat::Identity, Vec3::Ones, rad, mat));

    mat = new Material();
    mat->ambient = Vec3::Ones;
    mat->diffuse = Vec3::Ones;
    mat->phong = Vec3::Ones;
    mat->shininess = 100;
    mat->specular = Vec3::Ones;
    mat->refraction_index = 2;
    scene->materials.push_back(mat);

    scene->objects.push_back(
        new Sphere(Vec3(-(POX+PIX)/2, POY+rad, (POZ+PIZ)/2),
                   Quat::Identity, Vec3::Ones, rad, mat));
    scene->objects.push_back(
        new Sphere(Vec3((POX+PIX)/2, POY+rad, -(POZ+PIZ)/2),
                   Quat::Identity, Vec3::Ones, rad, mat));

    Light light;
    light.position = Vec3(-4, 8.5, 8) * 3;
    light.color = Vec3(.7,.7,.7);
    light.intensity = 500;
    scene->lights.push_back(light);
}

bool ldr_load_staff_scene(Scene* scene, int num)
{
    switch (num)
    {
    case 0:
        ldr_load_scene00(scene);
        break;
    case 1:
        ldr_load_scene01(scene);
        break;
    case 2:
        ldr_load_scene02(scene);
        break;
    default:
        return false;
        break;
    }

    return true;
}

